#region Copyright
/*
 * Copyright 2005-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using Seasar.Framework.Aop;
using Seasar.Framework.Aop.Proxy;
using Seasar.Framework.Container;
using Seasar.Framework.Container.Impl;
using Seasar.Quill.Exception;

namespace Seasar.Quill
{
    /// <summary>
    /// QuillContainerɊi[R|[lg`NX
    /// </summary>
    /// <remarks>
    /// RXgN^ŏsۂɃR|[lg̃IuWFNg
    /// CX^X
    /// </remarks>
    public class QuillComponent : IDisposable
    {
        /// <summary>
        /// R|[lgType
        /// </summary>
        protected Type componentType;

        /// <summary>
        /// R|[lg󂯎tB[hType
        /// </summary>
        protected Type receiptType;

        /// <summary>
        /// R|[lgObjecti[RNV
        /// </summary>
        /// <remarks>
        /// AspectKpꍇɂ̓vLVIuWFNgi[
        /// </remarks>
        protected Dictionary<Type, object> componentObjects = 
            new Dictionary<Type, object>(2);

        #region vpeB

        /// <summary>
        /// R|[lgType擾
        /// </summary>
        /// <value>R|[lgType</value>
        public Type ComponentType
        {
            get { return componentType; }
        }

        /// <summary>
        /// R|[lg󂯎tB[hType擾
        /// </summary>
        /// <value>R|[lg󂯎tB[hType</value>
        public Type ReceiptType
        {
            get { return receiptType; }
        }

        #endregion

        #region RXgN^

        /// <summary>
        /// QuillComponentRXgN^
        /// </summary>
        /// <param name="componentType">R|[lgType</param>
        /// <param name="receiptType">R|[lg󂯎tB[hType</param>
        /// <param name="aspects">Aspect`̔z</param>
        public QuillComponent(Type componentType, Type receiptType, IAspect[] aspects)
        {
            // R|[lgTypetB[hɃZbg
            this.componentType = componentType;

            // R|[lg󂯎tB[hTypetB[hɃZbg
            this.receiptType = receiptType;

            if (aspects.Length > 0)
            {
                // Aspect`ĂꍇProxyObject쐬
                CreateProxyObject(componentType, receiptType, aspects);
            }
            else
            {
                // Aspect`ĂȂꍇ͎NX̃CX^X쐬
                CreateObject(componentType, receiptType);
            }
        }

        #endregion

        #region public \bh

        /// <summary>
        /// R|[lg̃IuWFNg擾
        /// </summary>
        /// <param name="type">R|[lg󂯎tB[hType</param>
        /// <returns>
        /// R|[lg̃IuWFNg
        /// <para>typeɑΉIuWFNgȂꍇnullԂ</para>
        /// </returns>
        public virtual object GetComponentObject(Type type)
        {
            if (componentObjects == null)
            {
                // DestroyĂꍇ͗O𔭐
                throw new QuillApplicationException("EQLL0018");
            }

            if (componentObjects.ContainsKey(type))
            {
                // ΉIuWFNgi[Ăꍇ͕Ԃ
                return componentObjects[type];
            }
            else
            {
                // ΉIuWFNgi[ĂȂꍇnullԂ
                return null;
            }
        }

        /// <summary>
        /// QuillComponentQƂj
        /// </summary>
        public virtual void Destroy()
        {
            if (componentObjects == null)
            {
                return;
            }

            // A}l[W\[X
            Dispose();

            componentObjects[componentType] = null;

            if (componentType != null && !componentType.Equals(receiptType))
            {
                // 󂯑̌^قȂꍇ͎󂯑̌^Ŋi[ĂIuWFNgւ̎QƂj
                componentObjects[receiptType] = null;
            }

            componentObjects = null;
            componentType = null;
            receiptType = null;
        }

        #region IDisposable o

        /// <summary>
        /// R|[lgIDisposableĂꍇDisposeĂяo
        /// </summary>
        public virtual void Dispose()
        {
            if (componentObjects == null)
            {
                // DestroyĂꍇ͗O𔭐
                throw new QuillApplicationException("EQLL0018");
            }

            // IDisposableĂꍇ̓LXg(ĂȂꍇnull)
            IDisposable disposable = componentObjects[componentType] as IDisposable;

            if (disposable != null)
            {
                // IDisposableĂꍇDisposeĂяo
                disposable.Dispose();
            }
        }

        #endregion

        #endregion

        #region protected \bh

        /// <summary>
        /// NX̃CX^X쐬
        /// </summary>
        /// <remarks>
        /// CX^X쐬NXɂ̓p[^()publicł
        /// RXgN^KvƂ
        /// </remarks>
        /// <param name="componentType">R|[lgType</param>
        protected virtual void CreateObject(Type componentType, Type receiptType)
        {
            // RXgN^擾
            ConstructorInfo constructor = componentType.GetConstructor(new Type[] { });

            if (constructor == null)
            {
                // p[^publicłRXgN^Ȃꍇ͗OX[
                throw new QuillApplicationException(
                    "EQLL0017", new object[] { componentType.FullName });
            }

            // CX^X쐬
            object obj;
            try
            {
                obj = Activator.CreateInstance(componentType);
            }
            catch (System.Exception ex)
            {
                throw new QuillApplicationException(
                    "EQLL0036", new object[] {componentType.Name}, ex); 
            }

            // NX̌^Ŋi[
            componentObjects[componentType] = obj;

            if (!componentType.Equals(receiptType))
            {
                // 󂯑̌^قȂꍇ͎󂯑̌^Ŋi[
                componentObjects[receiptType] = obj;
            }
        }

        /// <summary>
        /// vLVIuWFNg쐬
        /// </summary>
        /// <param name="componentType">R|[lgType</param>
        /// <param name="receiptType">R|[lg󂯎tB[hType</param>
        /// <param name="aspects">KpAspect̔z</param>
        /// <returns>쐬ꂽvLVIuWFNg</returns>
        protected virtual void CreateProxyObject(
            Type componentType, Type receiptType, IAspect[] aspects)
        {
            // S2Container.NET̃R|[lg`쐬
            // S2Dao.NETIComponentDefDaoType擾Ă̂
            // ΉׁB(AƓK؂ȕ@őΉKv)
            IComponentDef componentDef = new ComponentDefImpl(componentType);

            // DynamicAopProxyɓñׂp[^
            Hashtable parameters = new Hashtable();

            // R|[lg`p[^ƂăZbg
            parameters[ContainerConstants.COMPONENT_DEF_NAME] = componentDef;

            // DynamicAopProxy쐬
            DynamicAopProxy aopProxy;
            try
            {
                aopProxy = new DynamicAopProxy(componentType, aspects, parameters);

                // ProxyObject쐬
                componentObjects[componentType] = aopProxy.Create();
            }
            catch (System.Exception ex)
            {
                throw new QuillApplicationException(
                    "EQLL0037", new object[] { componentType.Name }, ex);
            }

            if (!componentType.Equals(receiptType))
            {
                // R|[lǧ^ƃR|[lg󂯎tB[ȟ^
                // قȂꍇ͎󂯎ۂ̌^ɑΉProxyObject쐬
                componentObjects[receiptType] =
                    aopProxy.Create(receiptType, componentObjects[componentType]);
            }
        }

        #endregion
    }
}