#region Copyright
/*
 * Copyright 2005-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Reflection;
using Seasar.Framework.Util;

namespace Seasar.Extension.ADO.Impl
{
    public class DatabaseMetaDataImpl : IDatabaseMetaData
    {
        private readonly IDictionary _primaryKeys = new Hashtable(StringComparer.CurrentCultureIgnoreCase);
        private readonly IDictionary _columns = new Hashtable(StringComparer.CurrentCultureIgnoreCase);
        private readonly IDictionary _autoIncrementColumns = new Hashtable(StringComparer.CurrentCultureIgnoreCase);

        private DataSet _metaDataSet;

        private readonly IDataSource _dataSource;

        public DatabaseMetaDataImpl(IDataSource dataSource)
        {
            _dataSource = dataSource;
        }

        #region IDatabaseMetaData o

        public IList GetPrimaryKeySet(string tableName)
        {
            if (!_primaryKeys.Contains(tableName)) CreateTableMetaData(tableName);
            return (IList) _primaryKeys[tableName];
        }

        public IList GetColumnSet(string tableName)
        {
            if (!_columns.Contains(tableName)) CreateTableMetaData(tableName);
            return (IList) _columns[tableName];
        }

        public IList GetAutoIncrementColumnSet(string tableName)
        {
            if (!_autoIncrementColumns.Contains(tableName)) CreateTableMetaData(tableName);
            return (IList) _autoIncrementColumns[tableName];
        }

        #endregion

        private string _metaDataSetClassName;

        /// <summary>
        /// DB̃^i[<seealso cref="DataSet"/>̊SCݒ肷
        /// <seealso cref="DataSet"/>܂ރAZúA
        /// ݂̃AvP[VEhCɊ܂܂Kv
        /// </summary>
        public string MetaDataSetClassName
        {
            set { _metaDataSetClassName = value; }
        }

        /// <summary>
        /// e[u`쐬
        /// </summary>
        /// <param name="tableName">e[u</param>
        protected virtual void CreateTableMetaData(string tableName)
        {
            lock (this)
            {
                // e[u`擾
                DataTable metaDataTable;
                if (_metaDataSetClassName == null)
                {
                    metaDataTable = GetMetaDataForDatabase(tableName);
                }
                else
                {
                    metaDataTable = GetMetaDataForDataSet(tableName);
                }

                if (metaDataTable != null)
                {
                    // e[u`񂩂vC}L[擾
                    _primaryKeys[tableName] = GetPrimaryKeySet(metaDataTable.PrimaryKey);

                    // e[u`񂩂J擾
                    _columns[tableName] = GetColumnSet(metaDataTable.Columns);

                    // e[u`񂩂AutoIncrementJ擾
                    _autoIncrementColumns[tableName] = GetAutoIncrementColumnSet(metaDataTable.Columns);
                }
            }
        }

        /// <summary>
        /// DBe[u`擾
        /// </summary>
        /// <param name="tableName">e[u</param>
        /// <returns>e[u`B擾łȂꍇAnullԂ</returns>
        private DataTable GetMetaDataForDatabase(string tableName)
        {
            // e[u`
            DataTable[] metaDataTables;

            // IDbConnection擾
            IDbConnection cn = DataSourceUtil.GetConnection(_dataSource);
            try
            {
                // e[u`擾邽߂SQL쐬
                string sql = "SELECT * FROM " + tableName;

                // IDbCommand擾
                IDbCommand cmd = _dataSource.GetCommand(sql, cn);

                // Transaction̏s
                _dataSource.SetTransaction(cmd);

                // IDataAdapter擾
                IDataAdapter adapter = _dataSource.GetDataAdapter(cmd);

                // e[u`擾
                try
                {
                    metaDataTables = adapter.FillSchema(new DataSet(), SchemaType.Mapped);
                }
                catch
                {
                    return null;
                }
            }
            finally
            {
                // IDbConnectionCloses
                _dataSource.CloseConnection(cn);
            }

            return metaDataTables[0];
        }

        /// <summary>
        /// DB̃^i[<seealso cref="DataSet"/>e[u`擾
        /// </summary>
        /// <param name="tableName">e[u</param>
        /// <returns>e[u`B擾łȂꍇAnullԂ</returns>
        private DataTable GetMetaDataForDataSet(string tableName)
        {
            if (_metaDataSet == null)
            {
                Assembly[] loadedAssembly = AppDomain.CurrentDomain.GetAssemblies();
                Type dataSetType = ClassUtil.ForName(_metaDataSetClassName, loadedAssembly);
                _metaDataSet = (DataSet) ClassUtil.NewInstance(dataSetType);
            }

            if (!_metaDataSet.Tables.Contains(tableName))
            {
                return null;
            }
            else
            {
                return _metaDataSet.Tables[tableName];
            }
        }

        private IList GetPrimaryKeySet(IEnumerable<DataColumn> primarykeys)
        {
            IList list = new CaseInsentiveSet();
            foreach (DataColumn pkey in primarykeys)
            {
                list.Add(pkey.ColumnName);
            }
            return list;
        }

        private IList GetColumnSet(DataColumnCollection columns)
        {
            IList list = new CaseInsentiveSet();
            foreach (DataColumn column in columns)
            {
                list.Add(column.ColumnName);
            }
            return list;
        }

        private IList GetAutoIncrementColumnSet(DataColumnCollection columns)
        {
            IList list = new CaseInsentiveSet();
            foreach (DataColumn column in columns)
            {
                if (column.AutoIncrement)
                {
                    list.Add(column.ColumnName);
                }
            }
            return list;
        }
    }
}
